# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
import stripe
from stripe._expandable_field import ExpandableField
from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe._updateable_api_resource import UpdateableAPIResource
from stripe._util import sanitize_id
from typing import ClassVar, List, Optional
from typing_extensions import Literal, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._file import File


class Person(UpdateableAPIResource["Person"]):
    """
    This is an object representing a person associated with a Stripe account.

    A platform can only access a subset of data in a person for an account where [account.controller.requirement_collection](https://docs.stripe.com/api/accounts/object#account_object-controller-requirement_collection) is `stripe`, which includes Standard and Express accounts, after creating an Account Link or Account Session to start Connect onboarding.

    See the [Standard onboarding](https://docs.stripe.com/connect/standard-accounts) or [Express onboarding](https://docs.stripe.com/connect/express-accounts) documentation for information about prefilling information and account onboarding steps. Learn more about [handling identity verification with the API](https://docs.stripe.com/connect/handling-api-verification#person-information).
    """

    OBJECT_NAME: ClassVar[Literal["person"]] = "person"

    class AdditionalTosAcceptances(StripeObject):
        class Account(StripeObject):
            date: Optional[int]
            """
            The Unix timestamp marking when the legal guardian accepted the service agreement.
            """
            ip: Optional[str]
            """
            The IP address from which the legal guardian accepted the service agreement.
            """
            user_agent: Optional[str]
            """
            The user agent of the browser from which the legal guardian accepted the service agreement.
            """

        account: Optional[Account]
        """
        Details on the legal guardian's acceptance of the main Stripe service agreement.
        """
        _inner_class_types = {"account": Account}

    class Address(StripeObject):
        city: Optional[str]
        """
        City, district, suburb, town, or village.
        """
        country: Optional[str]
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: Optional[str]
        """
        Address line 1, such as the street, PO Box, or company name.
        """
        line2: Optional[str]
        """
        Address line 2, such as the apartment, suite, unit, or building.
        """
        postal_code: Optional[str]
        """
        ZIP or postal code.
        """
        state: Optional[str]
        """
        State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
        """

    class AddressKana(StripeObject):
        city: Optional[str]
        """
        City/Ward.
        """
        country: Optional[str]
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: Optional[str]
        """
        Block/Building number.
        """
        line2: Optional[str]
        """
        Building details.
        """
        postal_code: Optional[str]
        """
        ZIP or postal code.
        """
        state: Optional[str]
        """
        Prefecture.
        """
        town: Optional[str]
        """
        Town/cho-me.
        """

    class AddressKanji(StripeObject):
        city: Optional[str]
        """
        City/Ward.
        """
        country: Optional[str]
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: Optional[str]
        """
        Block/Building number.
        """
        line2: Optional[str]
        """
        Building details.
        """
        postal_code: Optional[str]
        """
        ZIP or postal code.
        """
        state: Optional[str]
        """
        Prefecture.
        """
        town: Optional[str]
        """
        Town/cho-me.
        """

    class BirthAddress(StripeObject):
        city: Optional[str]
        """
        City, district, suburb, town, or village.
        """
        country: Optional[str]
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: Optional[str]
        """
        Address line 1, such as the street, PO Box, or company name.
        """
        line2: Optional[str]
        """
        Address line 2, such as the apartment, suite, unit, or building.
        """
        postal_code: Optional[str]
        """
        ZIP or postal code.
        """
        state: Optional[str]
        """
        State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
        """

    class Dob(StripeObject):
        day: Optional[int]
        """
        The day of birth, between 1 and 31.
        """
        month: Optional[int]
        """
        The month of birth, between 1 and 12.
        """
        year: Optional[int]
        """
        The four-digit year of birth.
        """

    class FutureRequirements(StripeObject):
        class Alternative(StripeObject):
            alternative_fields_due: List[str]
            """
            Fields that can be provided to resolve all fields in `original_fields_due`.
            """
            original_fields_due: List[str]
            """
            Fields that are due and can be resolved by providing all fields in `alternative_fields_due`.
            """

        class Error(StripeObject):
            code: Literal[
                "external_request",
                "information_missing",
                "invalid_address_city_state_postal_code",
                "invalid_address_highway_contract_box",
                "invalid_address_private_mailbox",
                "invalid_business_profile_name",
                "invalid_business_profile_name_denylisted",
                "invalid_company_name_denylisted",
                "invalid_dob_age_over_maximum",
                "invalid_dob_age_under_18",
                "invalid_dob_age_under_minimum",
                "invalid_product_description_length",
                "invalid_product_description_url_match",
                "invalid_representative_country",
                "invalid_signator",
                "invalid_statement_descriptor_business_mismatch",
                "invalid_statement_descriptor_denylisted",
                "invalid_statement_descriptor_length",
                "invalid_statement_descriptor_prefix_denylisted",
                "invalid_statement_descriptor_prefix_mismatch",
                "invalid_street_address",
                "invalid_tax_id",
                "invalid_tax_id_format",
                "invalid_tos_acceptance",
                "invalid_url_denylisted",
                "invalid_url_format",
                "invalid_url_length",
                "invalid_url_web_presence_detected",
                "invalid_url_website_business_information_mismatch",
                "invalid_url_website_empty",
                "invalid_url_website_inaccessible",
                "invalid_url_website_inaccessible_geoblocked",
                "invalid_url_website_inaccessible_password_protected",
                "invalid_url_website_incomplete",
                "invalid_url_website_incomplete_cancellation_policy",
                "invalid_url_website_incomplete_customer_service_details",
                "invalid_url_website_incomplete_legal_restrictions",
                "invalid_url_website_incomplete_refund_policy",
                "invalid_url_website_incomplete_return_policy",
                "invalid_url_website_incomplete_terms_and_conditions",
                "invalid_url_website_incomplete_under_construction",
                "invalid_url_website_other",
                "invalid_value_other",
                "unsupported_business_type",
                "verification_data_not_found",
                "verification_directors_mismatch",
                "verification_document_address_mismatch",
                "verification_document_address_missing",
                "verification_document_corrupt",
                "verification_document_country_not_supported",
                "verification_document_directors_mismatch",
                "verification_document_dob_mismatch",
                "verification_document_duplicate_type",
                "verification_document_expired",
                "verification_document_failed_copy",
                "verification_document_failed_greyscale",
                "verification_document_failed_other",
                "verification_document_failed_test_mode",
                "verification_document_fraudulent",
                "verification_document_id_number_mismatch",
                "verification_document_id_number_missing",
                "verification_document_incomplete",
                "verification_document_invalid",
                "verification_document_issue_or_expiry_date_missing",
                "verification_document_manipulated",
                "verification_document_missing_back",
                "verification_document_missing_front",
                "verification_document_name_mismatch",
                "verification_document_name_missing",
                "verification_document_nationality_mismatch",
                "verification_document_not_readable",
                "verification_document_not_signed",
                "verification_document_not_uploaded",
                "verification_document_photo_mismatch",
                "verification_document_too_large",
                "verification_document_type_not_supported",
                "verification_extraneous_directors",
                "verification_failed_address_match",
                "verification_failed_authorizer_authority",
                "verification_failed_business_iec_number",
                "verification_failed_document_match",
                "verification_failed_id_number_match",
                "verification_failed_keyed_identity",
                "verification_failed_keyed_match",
                "verification_failed_name_match",
                "verification_failed_other",
                "verification_failed_representative_authority",
                "verification_failed_residential_address",
                "verification_failed_tax_id_match",
                "verification_failed_tax_id_not_issued",
                "verification_legal_entity_structure_mismatch",
                "verification_missing_directors",
                "verification_missing_executives",
                "verification_missing_owners",
                "verification_rejected_ownership_exemption_reason",
                "verification_requires_additional_memorandum_of_associations",
                "verification_requires_additional_proof_of_registration",
                "verification_supportability",
            ]
            """
            The code for the type of error.
            """
            reason: str
            """
            An informative message that indicates the error type and provides additional details about the error.
            """
            requirement: str
            """
            The specific user onboarding requirement field (in the requirements hash) that needs to be resolved.
            """

        alternatives: Optional[List[Alternative]]
        """
        Fields that are due and can be resolved by providing the corresponding alternative fields instead. Many alternatives can list the same `original_fields_due`, and any of these alternatives can serve as a pathway for attempting to resolve the fields again. Re-providing `original_fields_due` also serves as a pathway for attempting to resolve the fields again.
        """
        currently_due: List[str]
        """
        Fields that need to be resolved to keep the person's account enabled. If not resolved by the account's `future_requirements[current_deadline]`, these fields will transition to the main `requirements` hash, and may immediately become `past_due`, but the account may also be given a grace period depending on the account's enablement state prior to transition.
        """
        errors: List[Error]
        """
        Details about validation and verification failures for `due` requirements that must be resolved.
        """
        eventually_due: List[str]
        """
        Fields you must collect when all thresholds are reached. As they become required, they appear in `currently_due` as well, and the account's `future_requirements[current_deadline]` becomes set.
        """
        past_due: List[str]
        """
        Fields that haven't been resolved by the account's `requirements.current_deadline`. These fields need to be resolved to enable the person's account. `future_requirements.past_due` is a subset of `requirements.past_due`.
        """
        pending_verification: List[str]
        """
        Fields that are being reviewed, or might become required depending on the results of a review. If the review fails, these fields can move to `eventually_due`, `currently_due`, `past_due` or `alternatives`. Fields might appear in `eventually_due`, `currently_due`, `past_due` or `alternatives` and in `pending_verification` if one verification fails but another is still pending.
        """
        _inner_class_types = {"alternatives": Alternative, "errors": Error}

    class RegisteredAddress(StripeObject):
        city: Optional[str]
        """
        City, district, suburb, town, or village.
        """
        country: Optional[str]
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: Optional[str]
        """
        Address line 1, such as the street, PO Box, or company name.
        """
        line2: Optional[str]
        """
        Address line 2, such as the apartment, suite, unit, or building.
        """
        postal_code: Optional[str]
        """
        ZIP or postal code.
        """
        state: Optional[str]
        """
        State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
        """

    class Relationship(StripeObject):
        authorizer: Optional[bool]
        """
        Whether the person is the authorizer of the account's representative.
        """
        director: Optional[bool]
        """
        Whether the person is a director of the account's legal entity. Directors are typically members of the governing board of the company, or responsible for ensuring the company meets its regulatory obligations.
        """
        executive: Optional[bool]
        """
        Whether the person has significant responsibility to control, manage, or direct the organization.
        """
        legal_guardian: Optional[bool]
        """
        Whether the person is the legal guardian of the account's representative.
        """
        owner: Optional[bool]
        """
        Whether the person is an owner of the account's legal entity.
        """
        percent_ownership: Optional[float]
        """
        The percent owned by the person of the account's legal entity.
        """
        representative: Optional[bool]
        """
        Whether the person is authorized as the primary representative of the account. This is the person nominated by the business to provide information about themselves, and general information about the account. There can only be one representative at any given time. At the time the account is created, this person should be set to the person responsible for opening the account.
        """
        title: Optional[str]
        """
        The person's title (e.g., CEO, Support Engineer).
        """

    class Requirements(StripeObject):
        class Alternative(StripeObject):
            alternative_fields_due: List[str]
            """
            Fields that can be provided to resolve all fields in `original_fields_due`.
            """
            original_fields_due: List[str]
            """
            Fields that are due and can be resolved by providing all fields in `alternative_fields_due`.
            """

        class Error(StripeObject):
            code: Literal[
                "external_request",
                "information_missing",
                "invalid_address_city_state_postal_code",
                "invalid_address_highway_contract_box",
                "invalid_address_private_mailbox",
                "invalid_business_profile_name",
                "invalid_business_profile_name_denylisted",
                "invalid_company_name_denylisted",
                "invalid_dob_age_over_maximum",
                "invalid_dob_age_under_18",
                "invalid_dob_age_under_minimum",
                "invalid_product_description_length",
                "invalid_product_description_url_match",
                "invalid_representative_country",
                "invalid_signator",
                "invalid_statement_descriptor_business_mismatch",
                "invalid_statement_descriptor_denylisted",
                "invalid_statement_descriptor_length",
                "invalid_statement_descriptor_prefix_denylisted",
                "invalid_statement_descriptor_prefix_mismatch",
                "invalid_street_address",
                "invalid_tax_id",
                "invalid_tax_id_format",
                "invalid_tos_acceptance",
                "invalid_url_denylisted",
                "invalid_url_format",
                "invalid_url_length",
                "invalid_url_web_presence_detected",
                "invalid_url_website_business_information_mismatch",
                "invalid_url_website_empty",
                "invalid_url_website_inaccessible",
                "invalid_url_website_inaccessible_geoblocked",
                "invalid_url_website_inaccessible_password_protected",
                "invalid_url_website_incomplete",
                "invalid_url_website_incomplete_cancellation_policy",
                "invalid_url_website_incomplete_customer_service_details",
                "invalid_url_website_incomplete_legal_restrictions",
                "invalid_url_website_incomplete_refund_policy",
                "invalid_url_website_incomplete_return_policy",
                "invalid_url_website_incomplete_terms_and_conditions",
                "invalid_url_website_incomplete_under_construction",
                "invalid_url_website_other",
                "invalid_value_other",
                "unsupported_business_type",
                "verification_data_not_found",
                "verification_directors_mismatch",
                "verification_document_address_mismatch",
                "verification_document_address_missing",
                "verification_document_corrupt",
                "verification_document_country_not_supported",
                "verification_document_directors_mismatch",
                "verification_document_dob_mismatch",
                "verification_document_duplicate_type",
                "verification_document_expired",
                "verification_document_failed_copy",
                "verification_document_failed_greyscale",
                "verification_document_failed_other",
                "verification_document_failed_test_mode",
                "verification_document_fraudulent",
                "verification_document_id_number_mismatch",
                "verification_document_id_number_missing",
                "verification_document_incomplete",
                "verification_document_invalid",
                "verification_document_issue_or_expiry_date_missing",
                "verification_document_manipulated",
                "verification_document_missing_back",
                "verification_document_missing_front",
                "verification_document_name_mismatch",
                "verification_document_name_missing",
                "verification_document_nationality_mismatch",
                "verification_document_not_readable",
                "verification_document_not_signed",
                "verification_document_not_uploaded",
                "verification_document_photo_mismatch",
                "verification_document_too_large",
                "verification_document_type_not_supported",
                "verification_extraneous_directors",
                "verification_failed_address_match",
                "verification_failed_authorizer_authority",
                "verification_failed_business_iec_number",
                "verification_failed_document_match",
                "verification_failed_id_number_match",
                "verification_failed_keyed_identity",
                "verification_failed_keyed_match",
                "verification_failed_name_match",
                "verification_failed_other",
                "verification_failed_representative_authority",
                "verification_failed_residential_address",
                "verification_failed_tax_id_match",
                "verification_failed_tax_id_not_issued",
                "verification_legal_entity_structure_mismatch",
                "verification_missing_directors",
                "verification_missing_executives",
                "verification_missing_owners",
                "verification_rejected_ownership_exemption_reason",
                "verification_requires_additional_memorandum_of_associations",
                "verification_requires_additional_proof_of_registration",
                "verification_supportability",
            ]
            """
            The code for the type of error.
            """
            reason: str
            """
            An informative message that indicates the error type and provides additional details about the error.
            """
            requirement: str
            """
            The specific user onboarding requirement field (in the requirements hash) that needs to be resolved.
            """

        alternatives: Optional[List[Alternative]]
        """
        Fields that are due and can be resolved by providing the corresponding alternative fields instead. Many alternatives can list the same `original_fields_due`, and any of these alternatives can serve as a pathway for attempting to resolve the fields again. Re-providing `original_fields_due` also serves as a pathway for attempting to resolve the fields again.
        """
        currently_due: List[str]
        """
        Fields that need to be resolved to keep the person's account enabled. If not resolved by the account's `current_deadline`, these fields will appear in `past_due` as well, and the account is disabled.
        """
        errors: List[Error]
        """
        Details about validation and verification failures for `due` requirements that must be resolved.
        """
        eventually_due: List[str]
        """
        Fields you must collect when all thresholds are reached. As they become required, they appear in `currently_due` as well, and the account's `current_deadline` becomes set.
        """
        past_due: List[str]
        """
        Fields that haven't been resolved by `current_deadline`. These fields need to be resolved to enable the person's account.
        """
        pending_verification: List[str]
        """
        Fields that are being reviewed, or might become required depending on the results of a review. If the review fails, these fields can move to `eventually_due`, `currently_due`, `past_due` or `alternatives`. Fields might appear in `eventually_due`, `currently_due`, `past_due` or `alternatives` and in `pending_verification` if one verification fails but another is still pending.
        """
        _inner_class_types = {"alternatives": Alternative, "errors": Error}

    class SelfReportedIncome(StripeObject):
        amount: int
        """
        The amount in the minor currency unit (for example, cents for USD).
        """
        currency: str
        """
        Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
        """

    class SelfReportedMonthlyHousingPayment(StripeObject):
        amount: int
        """
        The amount in the minor currency unit (for example, cents for USD).
        """
        currency: str
        """
        Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
        """

    class UsCfpbData(StripeObject):
        class EthnicityDetails(StripeObject):
            ethnicity: Optional[
                List[
                    Literal[
                        "cuban",
                        "hispanic_or_latino",
                        "mexican",
                        "not_hispanic_or_latino",
                        "other_hispanic_or_latino",
                        "prefer_not_to_answer",
                        "puerto_rican",
                    ]
                ]
            ]
            """
            The persons ethnicity
            """
            ethnicity_other: Optional[str]
            """
            Please specify your origin, when other is selected.
            """

        class RaceDetails(StripeObject):
            race: Optional[
                List[
                    Literal[
                        "african_american",
                        "american_indian_or_alaska_native",
                        "asian",
                        "asian_indian",
                        "black_or_african_american",
                        "chinese",
                        "ethiopian",
                        "filipino",
                        "guamanian_or_chamorro",
                        "haitian",
                        "jamaican",
                        "japanese",
                        "korean",
                        "native_hawaiian",
                        "native_hawaiian_or_other_pacific_islander",
                        "nigerian",
                        "other_asian",
                        "other_black_or_african_american",
                        "other_pacific_islander",
                        "prefer_not_to_answer",
                        "samoan",
                        "somali",
                        "vietnamese",
                        "white",
                    ]
                ]
            ]
            """
            The persons race.
            """
            race_other: Optional[str]
            """
            Please specify your race, when other is selected.
            """

        ethnicity_details: Optional[EthnicityDetails]
        """
        The persons ethnicity details
        """
        race_details: Optional[RaceDetails]
        """
        The persons race details
        """
        self_identified_gender: Optional[str]
        """
        The persons self-identified gender
        """
        _inner_class_types = {
            "ethnicity_details": EthnicityDetails,
            "race_details": RaceDetails,
        }

    class Verification(StripeObject):
        class AdditionalDocument(StripeObject):
            back: Optional[ExpandableField["File"]]
            """
            The back of an ID returned by a [file upload](https://api.stripe.com#create_file) with a `purpose` value of `identity_document`.
            """
            details: Optional[str]
            """
            A user-displayable string describing the verification state of this document. For example, if a document is uploaded and the picture is too fuzzy, this may say "Identity document is too unclear to read".
            """
            details_code: Optional[str]
            """
            One of `document_corrupt`, `document_country_not_supported`, `document_expired`, `document_failed_copy`, `document_failed_other`, `document_failed_test_mode`, `document_fraudulent`, `document_failed_greyscale`, `document_incomplete`, `document_invalid`, `document_manipulated`, `document_missing_back`, `document_missing_front`, `document_not_readable`, `document_not_uploaded`, `document_photo_mismatch`, `document_too_large`, or `document_type_not_supported`. A machine-readable code specifying the verification state for this document.
            """
            front: Optional[ExpandableField["File"]]
            """
            The front of an ID returned by a [file upload](https://api.stripe.com#create_file) with a `purpose` value of `identity_document`.
            """

        class Document(StripeObject):
            back: Optional[ExpandableField["File"]]
            """
            The back of an ID returned by a [file upload](https://api.stripe.com#create_file) with a `purpose` value of `identity_document`.
            """
            details: Optional[str]
            """
            A user-displayable string describing the verification state of this document. For example, if a document is uploaded and the picture is too fuzzy, this may say "Identity document is too unclear to read".
            """
            details_code: Optional[str]
            """
            One of `document_corrupt`, `document_country_not_supported`, `document_expired`, `document_failed_copy`, `document_failed_other`, `document_failed_test_mode`, `document_fraudulent`, `document_failed_greyscale`, `document_incomplete`, `document_invalid`, `document_manipulated`, `document_missing_back`, `document_missing_front`, `document_not_readable`, `document_not_uploaded`, `document_photo_mismatch`, `document_too_large`, or `document_type_not_supported`. A machine-readable code specifying the verification state for this document.
            """
            front: Optional[ExpandableField["File"]]
            """
            The front of an ID returned by a [file upload](https://api.stripe.com#create_file) with a `purpose` value of `identity_document`.
            """

        additional_document: Optional[AdditionalDocument]
        """
        A document showing address, either a passport, local ID card, or utility bill from a well-known utility company.
        """
        details: Optional[str]
        """
        A user-displayable string describing the verification state for the person. For example, this may say "Provided identity information could not be verified".
        """
        details_code: Optional[str]
        """
        One of `document_address_mismatch`, `document_dob_mismatch`, `document_duplicate_type`, `document_id_number_mismatch`, `document_name_mismatch`, `document_nationality_mismatch`, `failed_keyed_identity`, or `failed_other`. A machine-readable code specifying the verification state for the person.
        """
        document: Optional[Document]
        status: str
        """
        The state of verification for the person. Possible values are `unverified`, `pending`, or `verified`. Please refer [guide](https://docs.stripe.com/connect/handling-api-verification) to handle verification updates.
        """
        _inner_class_types = {
            "additional_document": AdditionalDocument,
            "document": Document,
        }

    account: Optional[str]
    """
    The account the person is associated with.
    """
    additional_tos_acceptances: Optional[AdditionalTosAcceptances]
    address: Optional[Address]
    address_kana: Optional[AddressKana]
    """
    The Kana variation of the person's address (Japan only).
    """
    address_kanji: Optional[AddressKanji]
    """
    The Kanji variation of the person's address (Japan only).
    """
    birth_address: Optional[BirthAddress]
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    deleted: Optional[Literal[True]]
    """
    Always true for a deleted object
    """
    dob: Optional[Dob]
    email: Optional[str]
    """
    The person's email address. Also available for accounts where [controller.requirement_collection](https://docs.stripe.com/api/accounts/object#account_object-controller-requirement_collection) is `stripe`.
    """
    first_name: Optional[str]
    """
    The person's first name. Also available for accounts where [controller.requirement_collection](https://docs.stripe.com/api/accounts/object#account_object-controller-requirement_collection) is `stripe`.
    """
    first_name_kana: Optional[str]
    """
    The Kana variation of the person's first name (Japan only). Also available for accounts where [controller.requirement_collection](https://docs.stripe.com/api/accounts/object#account_object-controller-requirement_collection) is `stripe`.
    """
    first_name_kanji: Optional[str]
    """
    The Kanji variation of the person's first name (Japan only). Also available for accounts where [controller.requirement_collection](https://docs.stripe.com/api/accounts/object#account_object-controller-requirement_collection) is `stripe`.
    """
    full_name_aliases: Optional[List[str]]
    """
    A list of alternate names or aliases that the person is known by. Also available for accounts where [controller.requirement_collection](https://docs.stripe.com/api/accounts/object#account_object-controller-requirement_collection) is `stripe`.
    """
    future_requirements: Optional[FutureRequirements]
    """
    Information about the [upcoming new requirements for this person](https://docs.stripe.com/connect/custom-accounts/future-requirements), including what information needs to be collected, and by when.
    """
    gender: Optional[str]
    """
    The person's gender.
    """
    id: str
    """
    Unique identifier for the object.
    """
    id_number_provided: Optional[bool]
    """
    Whether the person's `id_number` was provided. True if either the full ID number was provided or if only the required part of the ID number was provided (ex. last four of an individual's SSN for the US indicated by `ssn_last_4_provided`).
    """
    id_number_secondary_provided: Optional[bool]
    """
    Whether the person's `id_number_secondary` was provided.
    """
    last_name: Optional[str]
    """
    The person's last name. Also available for accounts where [controller.requirement_collection](https://docs.stripe.com/api/accounts/object#account_object-controller-requirement_collection) is `stripe`.
    """
    last_name_kana: Optional[str]
    """
    The Kana variation of the person's last name (Japan only). Also available for accounts where [controller.requirement_collection](https://docs.stripe.com/api/accounts/object#account_object-controller-requirement_collection) is `stripe`.
    """
    last_name_kanji: Optional[str]
    """
    The Kanji variation of the person's last name (Japan only). Also available for accounts where [controller.requirement_collection](https://docs.stripe.com/api/accounts/object#account_object-controller-requirement_collection) is `stripe`.
    """
    maiden_name: Optional[str]
    """
    The person's maiden name.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    nationality: Optional[str]
    """
    The country where the person is a national.
    """
    object: Literal["person"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    phone: Optional[str]
    """
    The person's phone number.
    """
    political_exposure: Optional[Literal["existing", "none"]]
    """
    Indicates if the person or any of their representatives, family members, or other closely related persons, declares that they hold or have held an important public job or function, in any jurisdiction.
    """
    registered_address: Optional[RegisteredAddress]
    relationship: Optional[Relationship]
    requirements: Optional[Requirements]
    """
    Information about the requirements for this person, including what information needs to be collected, and by when.
    """
    self_reported_income: Optional[SelfReportedIncome]
    self_reported_monthly_housing_payment: Optional[
        SelfReportedMonthlyHousingPayment
    ]
    ssn_last_4_provided: Optional[bool]
    """
    Whether the last four digits of the person's Social Security number have been provided (U.S. only).
    """
    us_cfpb_data: Optional[UsCfpbData]
    """
    Demographic data related to the person.
    """
    verification: Optional[Verification]

    def instance_url(self):
        token = self.id
        account = self.account
        base = stripe.Account.class_url()
        assert account is not None
        acct_extn = sanitize_id(account)
        extn = sanitize_id(token)
        return "%s/%s/persons/%s" % (base, acct_extn, extn)

    @classmethod
    def modify(cls, sid, **params):
        raise NotImplementedError(
            "Can't modify a person without an account ID. "
            "Use stripe.Account.modify_person('account_id', 'person_id', ...) "
            "(see https://stripe.com/docs/api/persons/update)."
        )

    @classmethod
    def retrieve(cls, id, **params):
        raise NotImplementedError(
            "Can't retrieve a person without an account ID. "
            "Use stripe.Account.retrieve_person('account_id', 'person_id') "
            "(see https://stripe.com/docs/api/persons/retrieve)."
        )

    _inner_class_types = {
        "additional_tos_acceptances": AdditionalTosAcceptances,
        "address": Address,
        "address_kana": AddressKana,
        "address_kanji": AddressKanji,
        "birth_address": BirthAddress,
        "dob": Dob,
        "future_requirements": FutureRequirements,
        "registered_address": RegisteredAddress,
        "relationship": Relationship,
        "requirements": Requirements,
        "self_reported_income": SelfReportedIncome,
        "self_reported_monthly_housing_payment": SelfReportedMonthlyHousingPayment,
        "us_cfpb_data": UsCfpbData,
        "verification": Verification,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class FeatureUpdateParams(TypedDict):
    active: NotRequired[bool]
    """
    Inactive features cannot be attached to new products and will not be returned from the features list endpoint.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    metadata: NotRequired[
        "Literal['']|Dict[str, str]|UntypedStripeObject[str]"
    ]
    """
    Set of key-value pairs that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    name: NotRequired[str]
    """
    The feature's name, for your own purpose, not meant to be displayable to the customer.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class CollectionSettingUpdateParams(TypedDict):
    collection_method: NotRequired[Literal["automatic", "send_invoice"]]
    """
    Either automatic, or send_invoice. When charging automatically, Stripe attempts to pay this
    bill at the end of the period using the payment method attached to the billing profile. When sending an invoice,
    Stripe emails your billing profile an invoice with payment instructions.
    """
    display_name: NotRequired[str]
    """
    An optional customer-facing display name for the CollectionSetting object.
    To remove the display name, set it to an empty string in the request.
    Maximum length of 250 characters.
    """
    email_delivery: NotRequired["CollectionSettingUpdateParamsEmailDelivery"]
    """
    Email delivery settings.
    """
    live_version: NotRequired[str]
    """
    Optionally change the live version of the CollectionSetting. Billing Cadences and other objects that refer to this
    CollectionSetting uses this version when no overrides are set. Providing `live_version = "latest"` sets the
    CollectionSetting's `live_version` to its latest version.
    """
    lookup_key: NotRequired[str]
    """
    A lookup key used to retrieve settings dynamically from a static string.
    This may be up to 200 characters.
    """
    payment_method_configuration: NotRequired[str]
    """
    The ID of the PaymentMethodConfiguration object, which controls which payment methods are displayed to your customers.
    """
    payment_method_options: NotRequired[
        "CollectionSettingUpdateParamsPaymentMethodOptions"
    ]
    """
    Payment Method specific configuration to be stored on the object.
    """


class CollectionSettingUpdateParamsEmailDelivery(TypedDict):
    payment_due: NotRequired[
        "CollectionSettingUpdateParamsEmailDeliveryPaymentDue"
    ]
    """
    Controls emails for when the payment is due. For example after the invoice is finalized and transitions to Open state.
    """


class CollectionSettingUpdateParamsEmailDeliveryPaymentDue(TypedDict):
    enabled: bool
    """
    If true an email for the invoice would be generated and sent out.
    """
    include_payment_link: bool
    """
    If true the payment link to hosted invoice page would be included in email and PDF of the invoice.
    """


class CollectionSettingUpdateParamsPaymentMethodOptions(TypedDict):
    acss_debit: NotRequired[
        "CollectionSettingUpdateParamsPaymentMethodOptionsAcssDebit"
    ]
    """
    This sub-hash contains details about the Canadian pre-authorized debit payment method options.
    """
    bancontact: NotRequired[
        "CollectionSettingUpdateParamsPaymentMethodOptionsBancontact"
    ]
    """
    This sub-hash contains details about the Bancontact payment method.
    """
    card: NotRequired["CollectionSettingUpdateParamsPaymentMethodOptionsCard"]
    """
    This sub-hash contains details about the Card payment method options.
    """
    customer_balance: NotRequired[
        "CollectionSettingUpdateParamsPaymentMethodOptionsCustomerBalance"
    ]
    """
    This sub-hash contains details about the Bank transfer payment method options.
    """
    konbini: NotRequired[
        "CollectionSettingUpdateParamsPaymentMethodOptionsKonbini"
    ]
    """
    This sub-hash contains details about the Konbini payment method options.
    """
    sepa_debit: NotRequired[
        "CollectionSettingUpdateParamsPaymentMethodOptionsSepaDebit"
    ]
    """
    This sub-hash contains details about the SEPA Direct Debit payment method options.
    """
    us_bank_account: NotRequired[
        "CollectionSettingUpdateParamsPaymentMethodOptionsUsBankAccount"
    ]
    """
    This sub-hash contains details about the ACH direct debit payment method options.
    """


class CollectionSettingUpdateParamsPaymentMethodOptionsAcssDebit(TypedDict):
    mandate_options: NotRequired[
        "CollectionSettingUpdateParamsPaymentMethodOptionsAcssDebitMandateOptions"
    ]
    """
    Additional fields for Mandate creation.
    """
    verification_method: NotRequired[
        Literal["automatic", "instant", "microdeposits"]
    ]
    """
    Verification method.
    """


class CollectionSettingUpdateParamsPaymentMethodOptionsAcssDebitMandateOptions(
    TypedDict,
):
    transaction_type: NotRequired[Literal["business", "personal"]]
    """
    Transaction type of the mandate.
    """


class CollectionSettingUpdateParamsPaymentMethodOptionsBancontact(TypedDict):
    preferred_language: NotRequired[Literal["de", "en", "fr", "nl"]]
    """
    Preferred language of the Bancontact authorization page that the customer is redirected to.
    """


class CollectionSettingUpdateParamsPaymentMethodOptionsCard(TypedDict):
    mandate_options: NotRequired[
        "CollectionSettingUpdateParamsPaymentMethodOptionsCardMandateOptions"
    ]
    """
    Configuration options for setting up an eMandate for cards issued in India.
    """
    network: NotRequired[str]
    """
    Selected network to process the payment on. Depends on the available networks of the card.
    """
    request_three_d_secure: NotRequired[
        Literal["any", "automatic", "challenge"]
    ]
    """
    An advanced option 3D Secure. We strongly recommend that you rely on our SCA Engine to automatically prompt your customers
    for authentication based on risk level and [other requirements](https://docs.stripe.com/strong-customer-authentication).
    However, if you wish to request 3D Secure based on logic from your own fraud engine, provide this option.
    Read our guide on [manually requesting 3D Secure](https://docs.stripe.com/payments/3d-secure/authentication-flow#manual-three-ds) for more information on how this configuration interacts with Radar and our SCA Engine.
    """


class CollectionSettingUpdateParamsPaymentMethodOptionsCardMandateOptions(
    TypedDict,
):
    amount: NotRequired[int]
    """
    Amount to be charged for future payments.
    """
    amount_type: NotRequired[Literal["fixed", "maximum"]]
    """
    The AmountType for the mandate. One of `fixed` or `maximum`.
    """
    description: NotRequired[str]
    """
    A description of the mandate that is meant to be displayed to the customer.
    """


class CollectionSettingUpdateParamsPaymentMethodOptionsCustomerBalance(
    TypedDict,
):
    bank_transfer: NotRequired[
        "CollectionSettingUpdateParamsPaymentMethodOptionsCustomerBalanceBankTransfer"
    ]
    """
    Configuration for the bank transfer funding type, if the `funding_type` is set to `bank_transfer`.
    """
    funding_type: NotRequired[Literal["bank_transfer"]]
    """
    The funding method type to be used when there are not enough funds in the customer balance. Currently the only supported value is `bank_transfer`.
    """


class CollectionSettingUpdateParamsPaymentMethodOptionsCustomerBalanceBankTransfer(
    TypedDict,
):
    eu_bank_transfer: NotRequired[
        "CollectionSettingUpdateParamsPaymentMethodOptionsCustomerBalanceBankTransferEuBankTransfer"
    ]
    """
    Configuration for `eu_bank_transfer` funding type. Required if `type` is `eu_bank_transfer`.
    """
    type: NotRequired[
        Literal[
            "eu_bank_transfer",
            "gb_bank_transfer",
            "jp_bank_transfer",
            "mx_bank_transfer",
            "us_bank_transfer",
        ]
    ]
    """
    The bank transfer type that can be used for funding.
    """


class CollectionSettingUpdateParamsPaymentMethodOptionsCustomerBalanceBankTransferEuBankTransfer(
    TypedDict,
):
    country: Literal["BE", "DE", "ES", "FR", "IE", "NL"]
    """
    The desired country code of the bank account information.
    """


class CollectionSettingUpdateParamsPaymentMethodOptionsKonbini(TypedDict):
    pass


class CollectionSettingUpdateParamsPaymentMethodOptionsSepaDebit(TypedDict):
    pass


class CollectionSettingUpdateParamsPaymentMethodOptionsUsBankAccount(
    TypedDict
):
    financial_connections: "CollectionSettingUpdateParamsPaymentMethodOptionsUsBankAccountFinancialConnections"
    """
    Additional fields for Financial Connections Session creation.
    """
    verification_method: Literal["automatic", "instant", "microdeposits"]
    """
    Verification method.
    """


class CollectionSettingUpdateParamsPaymentMethodOptionsUsBankAccountFinancialConnections(
    TypedDict,
):
    filters: NotRequired[
        "CollectionSettingUpdateParamsPaymentMethodOptionsUsBankAccountFinancialConnectionsFilters"
    ]
    """
    Provide filters for the linked accounts that the customer can select for the payment method.
    """
    permissions: NotRequired[
        List[
            Literal["balances", "ownership", "payment_method", "transactions"]
        ]
    ]
    """
    The list of permissions to request. If this parameter is passed, the `payment_method` permission must be included.
    """
    prefetch: NotRequired[
        List[Literal["balances", "ownership", "transactions"]]
    ]
    """
    List of data features that you would like to retrieve upon account creation.
    """


class CollectionSettingUpdateParamsPaymentMethodOptionsUsBankAccountFinancialConnectionsFilters(
    TypedDict,
):
    account_subcategories: NotRequired[List[Literal["checking", "savings"]]]
    """
    The account subcategories to use to filter for selectable accounts.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._fx_quote import FxQuote
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.params._fx_quote_create_params import FxQuoteCreateParams
    from stripe.params._fx_quote_list_params import FxQuoteListParams
    from stripe.params._fx_quote_retrieve_params import FxQuoteRetrieveParams


class FxQuoteService(StripeService):
    def list(
        self,
        params: Optional["FxQuoteListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[FxQuote]":
        """
        Returns a list of FX quotes that have been issued. The FX quotes are returned in sorted order, with the most recent FX quotes appearing first.
        """
        return cast(
            "ListObject[FxQuote]",
            self._request(
                "get",
                "/v1/fx_quotes",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["FxQuoteListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[FxQuote]":
        """
        Returns a list of FX quotes that have been issued. The FX quotes are returned in sorted order, with the most recent FX quotes appearing first.
        """
        return cast(
            "ListObject[FxQuote]",
            await self._request_async(
                "get",
                "/v1/fx_quotes",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "FxQuoteCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "FxQuote":
        """
        Creates an FX Quote object
        """
        return cast(
            "FxQuote",
            self._request(
                "post",
                "/v1/fx_quotes",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "FxQuoteCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "FxQuote":
        """
        Creates an FX Quote object
        """
        return cast(
            "FxQuote",
            await self._request_async(
                "post",
                "/v1/fx_quotes",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["FxQuoteRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "FxQuote":
        """
        Retrieve an FX Quote object
        """
        return cast(
            "FxQuote",
            self._request(
                "get",
                "/v1/fx_quotes/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["FxQuoteRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "FxQuote":
        """
        Retrieve an FX Quote object
        """
        return cast(
            "FxQuote",
            await self._request_async(
                "get",
                "/v1/fx_quotes/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import NotRequired, TypedDict


class ClaimableSandboxRenewOnboardingLinkParams(TypedDict):
    onboarding_link_details: NotRequired[
        "ClaimableSandboxRenewOnboardingLinkParamsOnboardingLinkDetails"
    ]
    """
    Details about the onboarding link.
    If omitted, the existing onboarding link details will be reused.
    """


class ClaimableSandboxRenewOnboardingLinkParamsOnboardingLinkDetails(
    TypedDict
):
    refresh_url: str
    """
    The URL the user will be redirected to if the onboarding link is expired or invalid.
    The URL specified should attempt to generate a new onboarding link,
    and re-direct the user to this new onboarding link so that they can proceed with the onboarding flow.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.billing.contracts.pricing_lines._quantity_change_list_contract_pricing_line_quantity_changes_params import (
        QuantityChangeListContractPricingLineQuantityChangesParams,
    )
    from stripe.v2._list_object import ListObject
    from stripe.v2.billing._contract_pricing_line_quantity_change import (
        ContractPricingLineQuantityChange,
    )


class QuantityChangeService(StripeService):
    def list_contract_pricing_line_quantity_changes(
        self,
        contract_id: str,
        pricing_line_id: str,
        params: Optional[
            "QuantityChangeListContractPricingLineQuantityChangesParams"
        ] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[ContractPricingLineQuantityChange]":
        """
        List quantity changes for a pricing line on a contract.
        """
        return cast(
            "ListObject[ContractPricingLineQuantityChange]",
            self._request(
                "get",
                "/v2/billing/contracts/{contract_id}/pricing_lines/{pricing_line_id}/quantity_changes".format(
                    contract_id=sanitize_id(contract_id),
                    pricing_line_id=sanitize_id(pricing_line_id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_contract_pricing_line_quantity_changes_async(
        self,
        contract_id: str,
        pricing_line_id: str,
        params: Optional[
            "QuantityChangeListContractPricingLineQuantityChangesParams"
        ] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[ContractPricingLineQuantityChange]":
        """
        List quantity changes for a pricing line on a contract.
        """
        return cast(
            "ListObject[ContractPricingLineQuantityChange]",
            await self._request_async(
                "get",
                "/v2/billing/contracts/{contract_id}/pricing_lines/{pricing_line_id}/quantity_changes".format(
                    contract_id=sanitize_id(contract_id),
                    pricing_line_id=sanitize_id(pricing_line_id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe.v2._amount import AmountParam
from typing_extensions import NotRequired, TypedDict


class SettlementAllocationIntentUpdateParams(TypedDict):
    amount: NotRequired[AmountParam]
    """
    The new amount for the SettlementAllocationIntent. Only amount.value can be updated and currency must remain same.
    """
    reference: NotRequired[str]
    """
    The new reference for the SettlementAllocationIntent.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class MeterEventAdjustmentCreateParams(RequestOptions):
    cancel: NotRequired["MeterEventAdjustmentCreateParamsCancel"]
    """
    Specifies which event to cancel.
    """
    event_name: str
    """
    The name of the meter event. Corresponds with the `event_name` field on a meter.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    type: Literal["cancel"]
    """
    Specifies whether to cancel a single event or a range of events for a time period. Time period cancellation is not supported yet.
    """


class MeterEventAdjustmentCreateParamsCancel(TypedDict):
    identifier: NotRequired[str]
    """
    Unique identifier for the event. You can only cancel events within 24 hours of Stripe receiving them.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from decimal import Decimal
from stripe._createable_api_resource import CreateableAPIResource
from stripe._expandable_field import ExpandableField
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe._util import class_method_variant, sanitize_id
from typing import ClassVar, List, Optional, cast, overload
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._customer import Customer
    from stripe.billing._meter import Meter
    from stripe.params.billing._alert_activate_params import (
        AlertActivateParams,
    )
    from stripe.params.billing._alert_archive_params import AlertArchiveParams
    from stripe.params.billing._alert_create_params import AlertCreateParams
    from stripe.params.billing._alert_deactivate_params import (
        AlertDeactivateParams,
    )
    from stripe.params.billing._alert_list_params import AlertListParams
    from stripe.params.billing._alert_retrieve_params import (
        AlertRetrieveParams,
    )


class Alert(CreateableAPIResource["Alert"], ListableAPIResource["Alert"]):
    """
    A billing alert is a resource that notifies you when a certain usage threshold on a meter is crossed. For example, you might create a billing alert to notify you when a certain user made 100 API requests.
    """

    OBJECT_NAME: ClassVar[Literal["billing.alert"]] = "billing.alert"

    class CreditBalanceThreshold(StripeObject):
        class Filter(StripeObject):
            customer: Optional[ExpandableField["Customer"]]
            """
            Limit the scope of the alert to this customer ID
            """
            type: Literal["customer", "tenant"]

        class Lte(StripeObject):
            class CustomPricingUnit(StripeObject):
                class CustomPricingUnitDetails(StripeObject):
                    created: int
                    """
                    Time at which the object was created. Measured in seconds since the Unix epoch.
                    """
                    display_name: str
                    """
                    The name of the custom pricing unit.
                    """
                    id: str
                    """
                    Unique identifier for the object.
                    """
                    lookup_key: Optional[str]
                    """
                    A lookup key for the custom pricing unit.
                    """
                    metadata: UntypedStripeObject[str]
                    """
                    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
                    """
                    status: str
                    """
                    The status of the custom pricing unit.
                    """

                custom_pricing_unit_details: Optional[CustomPricingUnitDetails]
                """
                The custom pricing unit object.
                """
                id: str
                """
                Unique identifier for the object.
                """
                value: Decimal
                """
                A positive decimal string representing the amount.
                """
                _inner_class_types = {
                    "custom_pricing_unit_details": CustomPricingUnitDetails,
                }
                _field_encodings = {"value": "decimal_string"}

            class Monetary(StripeObject):
                currency: str
                """
                Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
                """
                value: int
                """
                A positive integer representing the amount.
                """

            balance_type: Literal["custom_pricing_unit", "monetary"]
            """
            The type of this balance. We currently only support `monetary` amounts.
            """
            custom_pricing_unit: Optional[CustomPricingUnit]
            """
            The custom pricing unit amount.
            """
            monetary: Optional[Monetary]
            """
            The monetary amount.
            """
            _inner_class_types = {
                "custom_pricing_unit": CustomPricingUnit,
                "monetary": Monetary,
            }

        filters: Optional[List[Filter]]
        """
        The filters allow limiting the scope of this credit balance alert. You must specify only a customer filter at this time.
        """
        lte: Lte
        _inner_class_types = {"filters": Filter, "lte": Lte}

    class SpendThreshold(StripeObject):
        class Filters(StripeObject):
            billing_cadence: Optional[str]
            """
            Filter by billing cadence ID.
            """
            pricing_plan: Optional[str]
            """
            Filter by pricing plan ID.
            """
            pricing_plan_subscription: Optional[str]
            """
            Filter by pricing plan subscription ID.
            """

        class Gte(StripeObject):
            class Amount(StripeObject):
                currency: str
                """
                Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
                """
                value: int
                """
                A positive integer representing the amount.
                """

            class CustomPricingUnit(StripeObject):
                class CustomPricingUnitDetails(StripeObject):
                    created: int
                    """
                    Time at which the object was created. Measured in seconds since the Unix epoch.
                    """
                    display_name: str
                    """
                    The name of the custom pricing unit.
                    """
                    id: str
                    """
                    Unique identifier for the object.
                    """
                    lookup_key: Optional[str]
                    """
                    A lookup key for the custom pricing unit.
                    """
                    metadata: UntypedStripeObject[str]
                    """
                    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
                    """
                    status: str
                    """
                    The status of the custom pricing unit.
                    """

                custom_pricing_unit_details: Optional[CustomPricingUnitDetails]
                """
                The custom pricing unit object.
                """
                id: str
                """
                Unique identifier for the object.
                """
                value: Decimal
                """
                A positive decimal string representing the amount.
                """
                _inner_class_types = {
                    "custom_pricing_unit_details": CustomPricingUnitDetails,
                }
                _field_encodings = {"value": "decimal_string"}

            amount: Optional[Amount]
            """
            The monetary amount. Present when type is `amount`.
            """
            custom_pricing_unit: Optional[CustomPricingUnit]
            """
            The custom pricing unit amount. Present when type is `custom_pricing_unit`.
            """
            type: Literal["amount", "custom_pricing_unit"]
            """
            The type of the threshold amount.
            """
            _inner_class_types = {
                "amount": Amount,
                "custom_pricing_unit": CustomPricingUnit,
            }

        aggregation_period: Literal["billing"]
        """
        Defines the period over which spend is aggregated.
        """
        filters: Optional[Filters]
        """
        Filters to scope the spend calculation.
        """
        group_by: Optional[
            Literal["billing_cadence", "pricing_plan_subscription"]
        ]
        """
        Defines the granularity of spend aggregation.
        """
        gte: Gte
        """
        The threshold value configuration for a spend threshold alert.
        """
        _inner_class_types = {"filters": Filters, "gte": Gte}

    class UsageThreshold(StripeObject):
        class Filter(StripeObject):
            customer: Optional[ExpandableField["Customer"]]
            """
            Limit the scope of the alert to this customer ID
            """
            type: Literal["customer"]

        filters: Optional[List[Filter]]
        """
        The filters allow limiting the scope of this usage alert. You can only specify up to one filter at this time.
        """
        gte: int
        """
        The value at which this alert will trigger.
        """
        meter: ExpandableField["Meter"]
        """
        The [Billing Meter](https://docs.stripe.com/api/billing/meter) ID whose usage is monitored.
        """
        recurrence: Literal["one_time"]
        """
        Defines how the alert will behave.
        """
        _inner_class_types = {"filters": Filter}

    alert_type: Literal[
        "credit_balance_threshold", "spend_threshold", "usage_threshold"
    ]
    """
    Defines the type of the alert.
    """
    credit_balance_threshold: Optional[CreditBalanceThreshold]
    """
    Encapsulates configuration of the alert to monitor billing credit balance.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    object: Literal["billing.alert"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    spend_threshold: Optional[SpendThreshold]
    """
    Encapsulates the alert's configuration to monitor spend on pricing plan subscriptions.
    """
    status: Optional[Literal["active", "archived", "inactive"]]
    """
    Status of the alert. This can be active, inactive or archived.
    """
    title: str
    """
    Title of the alert.
    """
    usage_threshold: Optional[UsageThreshold]
    """
    Encapsulates configuration of the alert to monitor usage on a specific [Billing Meter](https://docs.stripe.com/api/billing/meter).
    """

    @classmethod
    def _cls_activate(
        cls, id: str, **params: Unpack["AlertActivateParams"]
    ) -> "Alert":
        """
        Reactivates this alert, allowing it to trigger again.
        """
        return cast(
            "Alert",
            cls._static_request(
                "post",
                "/v1/billing/alerts/{id}/activate".format(id=sanitize_id(id)),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def activate(id: str, **params: Unpack["AlertActivateParams"]) -> "Alert":
        """
        Reactivates this alert, allowing it to trigger again.
        """
        ...

    @overload
    def activate(self, **params: Unpack["AlertActivateParams"]) -> "Alert":
        """
        Reactivates this alert, allowing it to trigger again.
        """
        ...

    @class_method_variant("_cls_activate")
    def activate(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["AlertActivateParams"]
    ) -> "Alert":
        """
        Reactivates this alert, allowing it to trigger again.
        """
        return cast(
            "Alert",
            self._request(
                "post",
                "/v1/billing/alerts/{id}/activate".format(
                    id=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_activate_async(
        cls, id: str, **params: Unpack["AlertActivateParams"]
    ) -> "Alert":
        """
        Reactivates this alert, allowing it to trigger again.
        """
        return cast(
            "Alert",
            await cls._static_request_async(
                "post",
                "/v1/billing/alerts/{id}/activate".format(id=sanitize_id(id)),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def activate_async(
        id: str, **params: Unpack["AlertActivateParams"]
    ) -> "Alert":
        """
        Reactivates this alert, allowing it to trigger again.
        """
        ...

    @overload
    async def activate_async(
        self, **params: Unpack["AlertActivateParams"]
    ) -> "Alert":
        """
        Reactivates this alert, allowing it to trigger again.
        """
        ...

    @class_method_variant("_cls_activate_async")
    async def activate_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["AlertActivateParams"]
    ) -> "Alert":
        """
        Reactivates this alert, allowing it to trigger again.
        """
        return cast(
            "Alert",
            await self._request_async(
                "post",
                "/v1/billing/alerts/{id}/activate".format(
                    id=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def _cls_archive(
        cls, id: str, **params: Unpack["AlertArchiveParams"]
    ) -> "Alert":
        """
        Archives this alert, removing it from the list view and APIs. This is non-reversible.
        """
        return cast(
            "Alert",
            cls._static_request(
                "post",
                "/v1/billing/alerts/{id}/archive".format(id=sanitize_id(id)),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def archive(id: str, **params: Unpack["AlertArchiveParams"]) -> "Alert":
        """
        Archives this alert, removing it from the list view and APIs. This is non-reversible.
        """
        ...

    @overload
    def archive(self, **params: Unpack["AlertArchiveParams"]) -> "Alert":
        """
        Archives this alert, removing it from the list view and APIs. This is non-reversible.
        """
        ...

    @class_method_variant("_cls_archive")
    def archive(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["AlertArchiveParams"]
    ) -> "Alert":
        """
        Archives this alert, removing it from the list view and APIs. This is non-reversible.
        """
        return cast(
            "Alert",
            self._request(
                "post",
                "/v1/billing/alerts/{id}/archive".format(
                    id=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_archive_async(
        cls, id: str, **params: Unpack["AlertArchiveParams"]
    ) -> "Alert":
        """
        Archives this alert, removing it from the list view and APIs. This is non-reversible.
        """
        return cast(
            "Alert",
            await cls._static_request_async(
                "post",
                "/v1/billing/alerts/{id}/archive".format(id=sanitize_id(id)),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def archive_async(
        id: str, **params: Unpack["AlertArchiveParams"]
    ) -> "Alert":
        """
        Archives this alert, removing it from the list view and APIs. This is non-reversible.
        """
        ...

    @overload
    async def archive_async(
        self, **params: Unpack["AlertArchiveParams"]
    ) -> "Alert":
        """
        Archives this alert, removing it from the list view and APIs. This is non-reversible.
        """
        ...

    @class_method_variant("_cls_archive_async")
    async def archive_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["AlertArchiveParams"]
    ) -> "Alert":
        """
        Archives this alert, removing it from the list view and APIs. This is non-reversible.
        """
        return cast(
            "Alert",
            await self._request_async(
                "post",
                "/v1/billing/alerts/{id}/archive".format(
                    id=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def create(cls, **params: Unpack["AlertCreateParams"]) -> "Alert":
        """
        Creates a billing alert
        """
        return cast(
            "Alert",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["AlertCreateParams"]
    ) -> "Alert":
        """
        Creates a billing alert
        """
        return cast(
            "Alert",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def _cls_deactivate(
        cls, id: str, **params: Unpack["AlertDeactivateParams"]
    ) -> "Alert":
        """
        Deactivates this alert, preventing it from triggering.
        """
        return cast(
            "Alert",
            cls._static_request(
                "post",
                "/v1/billing/alerts/{id}/deactivate".format(
                    id=sanitize_id(id)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def deactivate(
        id: str, **params: Unpack["AlertDeactivateParams"]
    ) -> "Alert":
        """
        Deactivates this alert, preventing it from triggering.
        """
        ...

    @overload
    def deactivate(self, **params: Unpack["AlertDeactivateParams"]) -> "Alert":
        """
        Deactivates this alert, preventing it from triggering.
        """
        ...

    @class_method_variant("_cls_deactivate")
    def deactivate(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["AlertDeactivateParams"]
    ) -> "Alert":
        """
        Deactivates this alert, preventing it from triggering.
        """
        return cast(
            "Alert",
            self._request(
                "post",
                "/v1/billing/alerts/{id}/deactivate".format(
                    id=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_deactivate_async(
        cls, id: str, **params: Unpack["AlertDeactivateParams"]
    ) -> "Alert":
        """
        Deactivates this alert, preventing it from triggering.
        """
        return cast(
            "Alert",
            await cls._static_request_async(
                "post",
                "/v1/billing/alerts/{id}/deactivate".format(
                    id=sanitize_id(id)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def deactivate_async(
        id: str, **params: Unpack["AlertDeactivateParams"]
    ) -> "Alert":
        """
        Deactivates this alert, preventing it from triggering.
        """
        ...

    @overload
    async def deactivate_async(
        self, **params: Unpack["AlertDeactivateParams"]
    ) -> "Alert":
        """
        Deactivates this alert, preventing it from triggering.
        """
        ...

    @class_method_variant("_cls_deactivate_async")
    async def deactivate_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["AlertDeactivateParams"]
    ) -> "Alert":
        """
        Deactivates this alert, preventing it from triggering.
        """
        return cast(
            "Alert",
            await self._request_async(
                "post",
                "/v1/billing/alerts/{id}/deactivate".format(
                    id=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def list(cls, **params: Unpack["AlertListParams"]) -> ListObject["Alert"]:
        """
        Lists billing active and inactive alerts
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["AlertListParams"]
    ) -> ListObject["Alert"]:
        """
        Lists billing active and inactive alerts
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["AlertRetrieveParams"]
    ) -> "Alert":
        """
        Retrieves a billing alert given an ID
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["AlertRetrieveParams"]
    ) -> "Alert":
        """
        Retrieves a billing alert given an ID
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    _inner_class_types = {
        "credit_balance_threshold": CreditBalanceThreshold,
        "spend_threshold": SpendThreshold,
        "usage_threshold": UsageThreshold,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
import json
from stripe._api_version import _ApiVersion
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from uuid import uuid4
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._discount import Discount
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe._search_result_object import SearchResultObject
    from stripe._subscription import Subscription
    from stripe.params._subscription_attach_cadence_params import (
        SubscriptionAttachCadenceParams,
    )
    from stripe.params._subscription_cancel_params import (
        SubscriptionCancelParams,
    )
    from stripe.params._subscription_create_params import (
        SubscriptionCreateParams,
    )
    from stripe.params._subscription_delete_discount_params import (
        SubscriptionDeleteDiscountParams,
    )
    from stripe.params._subscription_list_params import SubscriptionListParams
    from stripe.params._subscription_migrate_params import (
        SubscriptionMigrateParams,
    )
    from stripe.params._subscription_pause_params import (
        SubscriptionPauseParams,
    )
    from stripe.params._subscription_resume_params import (
        SubscriptionResumeParams,
    )
    from stripe.params._subscription_retrieve_params import (
        SubscriptionRetrieveParams,
    )
    from stripe.params._subscription_search_params import (
        SubscriptionSearchParams,
    )
    from stripe.params._subscription_update_params import (
        SubscriptionUpdateParams,
    )


class SubscriptionService(StripeService):
    def cancel(
        self,
        subscription_exposed_id: str,
        params: Optional["SubscriptionCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Subscription":
        """
        Cancels a customer's subscription immediately. The customer won't be charged again for the subscription. After it's canceled, the subscription is largely immutable. You can still update its [metadata](https://docs.stripe.com/metadata) and cancellation_details.

        Any pending invoice items that you've created are still charged at the end of the period, unless manually [deleted](https://docs.stripe.com/api/invoiceitems/delete). If you've set the subscription to cancel at the end of the period, any pending prorations are also left in place and collected at the end of the period. But if the subscription is set to cancel immediately, pending prorations are removed if invoice_now and prorate are both set to false.

        By default, upon subscription cancellation, Stripe stops automatic collection of all finalized invoices for the customer. This is intended to prevent unexpected payment attempts after the customer has canceled a subscription. However, you can resume automatic collection of the invoices manually after subscription cancellation to have us proceed. Or, you could check for unpaid invoices before allowing the customer to cancel the subscription at all.
        """
        return cast(
            "Subscription",
            self._request(
                "delete",
                "/v1/subscriptions/{subscription_exposed_id}".format(
                    subscription_exposed_id=sanitize_id(
                        subscription_exposed_id
                    ),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def cancel_async(
        self,
        subscription_exposed_id: str,
        params: Optional["SubscriptionCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Subscription":
        """
        Cancels a customer's subscription immediately. The customer won't be charged again for the subscription. After it's canceled, the subscription is largely immutable. You can still update its [metadata](https://docs.stripe.com/metadata) and cancellation_details.

        Any pending invoice items that you've created are still charged at the end of the period, unless manually [deleted](https://docs.stripe.com/api/invoiceitems/delete). If you've set the subscription to cancel at the end of the period, any pending prorations are also left in place and collected at the end of the period. But if the subscription is set to cancel immediately, pending prorations are removed if invoice_now and prorate are both set to false.

        By default, upon subscription cancellation, Stripe stops automatic collection of all finalized invoices for the customer. This is intended to prevent unexpected payment attempts after the customer has canceled a subscription. However, you can resume automatic collection of the invoices manually after subscription cancellation to have us proceed. Or, you could check for unpaid invoices before allowing the customer to cancel the subscription at all.
        """
        return cast(
            "Subscription",
            await self._request_async(
                "delete",
                "/v1/subscriptions/{subscription_exposed_id}".format(
                    subscription_exposed_id=sanitize_id(
                        subscription_exposed_id
                    ),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        subscription_exposed_id: str,
        params: Optional["SubscriptionRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Subscription":
        """
        Retrieves the subscription with the given ID.
        """
        return cast(
            "Subscription",
            self._request(
                "get",
                "/v1/subscriptions/{subscription_exposed_id}".format(
                    subscription_exposed_id=sanitize_id(
                        subscription_exposed_id
                    ),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        subscription_exposed_id: str,
        params: Optional["SubscriptionRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Subscription":
        """
        Retrieves the subscription with the given ID.
        """
        return cast(
            "Subscription",
            await self._request_async(
                "get",
                "/v1/subscriptions/{subscription_exposed_id}".format(
                    subscription_exposed_id=sanitize_id(
                        subscription_exposed_id
                    ),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        subscription_exposed_id: str,
        params: Optional["SubscriptionUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Subscription":
        """
        Updates an existing subscription to match the specified parameters.
        When changing prices or quantities, we optionally prorate the price we charge next month to make up for any price changes.
        To preview how the proration is calculated, use the [create preview](https://docs.stripe.com/docs/api/invoices/create_preview) endpoint.

        By default, we prorate subscription changes. For example, if a customer signs up on May 1 for a 100 price, they'll be billed 100 immediately. If on May 15 they switch to a 200 price, then on June 1 they'll be billed 250 (200 for a renewal of her subscription, plus a 50 prorating adjustment for half of the previous month's 100 difference). Similarly, a downgrade generates a credit that is applied to the next invoice. We also prorate when you make quantity changes.

        Switching prices does not normally change the billing date or generate an immediate charge unless:


        The billing interval is changed (for example, from monthly to yearly).
        The subscription moves from free to paid.
        A trial starts or ends.


        In these cases, we apply a credit for the unused time on the previous price, immediately charge the customer using the new price, and reset the billing date. Learn about how [Stripe immediately attempts payment for subscription changes](https://docs.stripe.com/docs/billing/subscriptions/upgrade-downgrade#immediate-payment).

        If you want to charge for an upgrade immediately, pass proration_behavior as always_invoice to create prorations, automatically invoice the customer for those proration adjustments, and attempt to collect payment. If you pass create_prorations, the prorations are created but not automatically invoiced. If you want to bill the customer for the prorations before the subscription's renewal date, you need to manually [invoice the customer](https://docs.stripe.com/docs/api/invoices/create).

        If you don't want to prorate, set the proration_behavior option to none. With this option, the customer is billed 100 on May 1 and 200 on June 1. Similarly, if you set proration_behavior to none when switching between different billing intervals (for example, from monthly to yearly), we don't generate any credits for the old subscription's unused time. We still reset the billing date and bill immediately for the new subscription.

        Updating the quantity on a subscription many times in an hour may result in [rate limiting. If you need to bill for a frequently changing quantity, consider integrating <a href="/docs/billing/subscriptions/usage-based">usage-based billing](https://docs.stripe.com/docs/rate-limits) instead.
        """
        return cast(
            "Subscription",
            self._request(
                "post",
                "/v1/subscriptions/{subscription_exposed_id}".format(
                    subscription_exposed_id=sanitize_id(
                        subscription_exposed_id
                    ),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        subscription_exposed_id: str,
        params: Optional["SubscriptionUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Subscription":
        """
        Updates an existing subscription to match the specified parameters.
        When changing prices or quantities, we optionally prorate the price we charge next month to make up for any price changes.
        To preview how the proration is calculated, use the [create preview](https://docs.stripe.com/docs/api/invoices/create_preview) endpoint.

        By default, we prorate subscription changes. For example, if a customer signs up on May 1 for a 100 price, they'll be billed 100 immediately. If on May 15 they switch to a 200 price, then on June 1 they'll be billed 250 (200 for a renewal of her subscription, plus a 50 prorating adjustment for half of the previous month's 100 difference). Similarly, a downgrade generates a credit that is applied to the next invoice. We also prorate when you make quantity changes.

        Switching prices does not normally change the billing date or generate an immediate charge unless:


        The billing interval is changed (for example, from monthly to yearly).
        The subscription moves from free to paid.
        A trial starts or ends.


        In these cases, we apply a credit for the unused time on the previous price, immediately charge the customer using the new price, and reset the billing date. Learn about how [Stripe immediately attempts payment for subscription changes](https://docs.stripe.com/docs/billing/subscriptions/upgrade-downgrade#immediate-payment).

        If you want to charge for an upgrade immediately, pass proration_behavior as always_invoice to create prorations, automatically invoice the customer for those proration adjustments, and attempt to collect payment. If you pass create_prorations, the prorations are created but not automatically invoiced. If you want to bill the customer for the prorations before the subscription's renewal date, you need to manually [invoice the customer](https://docs.stripe.com/docs/api/invoices/create).

        If you don't want to prorate, set the proration_behavior option to none. With this option, the customer is billed 100 on May 1 and 200 on June 1. Similarly, if you set proration_behavior to none when switching between different billing intervals (for example, from monthly to yearly), we don't generate any credits for the old subscription's unused time. We still reset the billing date and bill immediately for the new subscription.

        Updating the quantity on a subscription many times in an hour may result in [rate limiting. If you need to bill for a frequently changing quantity, consider integrating <a href="/docs/billing/subscriptions/usage-based">usage-based billing](https://docs.stripe.com/docs/rate-limits) instead.
        """
        return cast(
            "Subscription",
            await self._request_async(
                "post",
                "/v1/subscriptions/{subscription_exposed_id}".format(
                    subscription_exposed_id=sanitize_id(
                        subscription_exposed_id
                    ),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def delete_discount(
        self,
        subscription_exposed_id: str,
        params: Optional["SubscriptionDeleteDiscountParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Discount":
        """
        Removes the currently applied discount on a subscription.
        """
        return cast(
            "Discount",
            self._request(
                "delete",
                "/v1/subscriptions/{subscription_exposed_id}/discount".format(
                    subscription_exposed_id=sanitize_id(
                        subscription_exposed_id
                    ),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def delete_discount_async(
        self,
        subscription_exposed_id: str,
        params: Optional["SubscriptionDeleteDiscountParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Discount":
        """
        Removes the currently applied discount on a subscription.
        """
        return cast(
            "Discount",
            await self._request_async(
                "delete",
                "/v1/subscriptions/{subscription_exposed_id}/discount".format(
                    subscription_exposed_id=sanitize_id(
                        subscription_exposed_id
                    ),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def list(
        self,
        params: Optional["SubscriptionListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Subscription]":
        """
        By default, returns a list of subscriptions that have not been canceled. In order to list canceled subscriptions, specify status=canceled.
        """
        return cast(
            "ListObject[Subscription]",
            self._request(
                "get",
                "/v1/subscriptions",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["SubscriptionListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Subscription]":
        """
        By default, returns a list of subscriptions that have not been canceled. In order to list canceled subscriptions, specify status=canceled.
        """
        return cast(
            "ListObject[Subscription]",
            await self._request_async(
                "get",
                "/v1/subscriptions",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: Optional["SubscriptionCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Subscription":
        """
        Creates a new subscription on an existing customer. Each customer can have up to 500 active or scheduled subscriptions.

        When you create a subscription with collection_method=charge_automatically, the first invoice is finalized as part of the request.
        The payment_behavior parameter determines the exact behavior of the initial payment.

        To start subscriptions where the first invoice always begins in a draft status, use [subscription schedules](https://docs.stripe.com/docs/billing/subscriptions/subscription-schedules#managing) instead.
        Schedules provide the flexibility to model more complex billing configurations that change over time.
        """
        return cast(
            "Subscription",
            self._request(
                "post",
                "/v1/subscriptions",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: Optional["SubscriptionCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Subscription":
        """
        Creates a new subscription on an existing customer. Each customer can have up to 500 active or scheduled subscriptions.

        When you create a subscription with collection_method=charge_automatically, the first invoice is finalized as part of the request.
        The payment_behavior parameter determines the exact behavior of the initial payment.

        To start subscriptions where the first invoice always begins in a draft status, use [subscription schedules](https://docs.stripe.com/docs/billing/subscriptions/subscription-schedules#managing) instead.
        Schedules provide the flexibility to model more complex billing configurations that change over time.
        """
        return cast(
            "Subscription",
            await self._request_async(
                "post",
                "/v1/subscriptions",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def search(
        self,
        params: "SubscriptionSearchParams",
        options: Optional["RequestOptions"] = None,
    ) -> "SearchResultObject[Subscription]":
        """
        Search for subscriptions you've previously created using Stripe's [Search Query Language](https://docs.stripe.com/docs/search#search-query-language).
        Don't use search in read-after-write flows where strict consistency is necessary. Under normal operating
        conditions, data is searchable in less than a minute. Occasionally, propagation of new or updated data can be up
        to an hour behind during outages. Search functionality is not available to merchants in India.
        """
        return cast(
            "SearchResultObject[Subscription]",
            self._request(
                "get",
                "/v1/subscriptions/search",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def search_async(
        self,
        params: "SubscriptionSearchParams",
        options: Optional["RequestOptions"] = None,
    ) -> "SearchResultObject[Subscription]":
        """
        Search for subscriptions you've previously created using Stripe's [Search Query Language](https://docs.stripe.com/docs/search#search-query-language).
        Don't use search in read-after-write flows where strict consistency is necessary. Under normal operating
        conditions, data is searchable in less than a minute. Occasionally, propagation of new or updated data can be up
        to an hour behind during outages. Search functionality is not available to merchants in India.
        """
        return cast(
            "SearchResultObject[Subscription]",
            await self._request_async(
                "get",
                "/v1/subscriptions/search",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def attach_cadence(
        self,
        subscription: str,
        params: "SubscriptionAttachCadenceParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Subscription":
        """
        Attach a Billing Cadence to an existing subscription. When attached, the subscription is billed by the Billing Cadence, potentially sharing invoices with the other subscriptions linked to the Billing Cadence.
        """
        return cast(
            "Subscription",
            self._request(
                "post",
                "/v1/subscriptions/{subscription}/attach_cadence".format(
                    subscription=sanitize_id(subscription),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def attach_cadence_async(
        self,
        subscription: str,
        params: "SubscriptionAttachCadenceParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Subscription":
        """
        Attach a Billing Cadence to an existing subscription. When attached, the subscription is billed by the Billing Cadence, potentially sharing invoices with the other subscriptions linked to the Billing Cadence.
        """
        return cast(
            "Subscription",
            await self._request_async(
                "post",
                "/v1/subscriptions/{subscription}/attach_cadence".format(
                    subscription=sanitize_id(subscription),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def migrate(
        self,
        subscription: str,
        params: "SubscriptionMigrateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Subscription":
        """
        Upgrade the billing_mode of an existing subscription.
        """
        return cast(
            "Subscription",
            self._request(
                "post",
                "/v1/subscriptions/{subscription}/migrate".format(
                    subscription=sanitize_id(subscription),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def migrate_async(
        self,
        subscription: str,
        params: "SubscriptionMigrateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Subscription":
        """
        Upgrade the billing_mode of an existing subscription.
        """
        return cast(
            "Subscription",
            await self._request_async(
                "post",
                "/v1/subscriptions/{subscription}/migrate".format(
                    subscription=sanitize_id(subscription),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def pause(
        self,
        subscription: str,
        params: Optional["SubscriptionPauseParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Subscription":
        """
        Pauses a subscription by transitioning it to the paused status. A paused subscription does not generate invoices and will not advance to new billing periods. The subscription can be resumed later using the resume endpoint. Cannot pause subscriptions with attached schedules.
        """
        return cast(
            "Subscription",
            self._request(
                "post",
                "/v1/subscriptions/{subscription}/pause".format(
                    subscription=sanitize_id(subscription),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def pause_async(
        self,
        subscription: str,
        params: Optional["SubscriptionPauseParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Subscription":
        """
        Pauses a subscription by transitioning it to the paused status. A paused subscription does not generate invoices and will not advance to new billing periods. The subscription can be resumed later using the resume endpoint. Cannot pause subscriptions with attached schedules.
        """
        return cast(
            "Subscription",
            await self._request_async(
                "post",
                "/v1/subscriptions/{subscription}/pause".format(
                    subscription=sanitize_id(subscription),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def resume(
        self,
        subscription: str,
        params: Optional["SubscriptionResumeParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Subscription":
        """
        Initiates resumption of a paused subscription, optionally resetting the billing cycle anchor and creating prorations. Resume is only available for subscriptions that use charge_automatically collection. If Stripe doesn't generate a resumption invoice, the subscription becomes active immediately. When a resumption invoice is generated, Stripe finalizes it immediately. If the invoice is paid or marked uncollectible, the subscription becomes active. If the invoice is manually voided, the subscription stays paused. If there is no payment attempt within 23 hours, Stripe voids the invoice and the subscription stays paused. Learn more about [resuming subscriptions](https://docs.stripe.com/docs/billing/subscriptions/pause#resume-subscriptions).
        """
        return cast(
            "Subscription",
            self._request(
                "post",
                "/v1/subscriptions/{subscription}/resume".format(
                    subscription=sanitize_id(subscription),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def resume_async(
        self,
        subscription: str,
        params: Optional["SubscriptionResumeParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Subscription":
        """
        Initiates resumption of a paused subscription, optionally resetting the billing cycle anchor and creating prorations. Resume is only available for subscriptions that use charge_automatically collection. If Stripe doesn't generate a resumption invoice, the subscription becomes active immediately. When a resumption invoice is generated, Stripe finalizes it immediately. If the invoice is paid or marked uncollectible, the subscription becomes active. If the invoice is manually voided, the subscription stays paused. If there is no payment attempt within 23 hours, Stripe voids the invoice and the subscription stays paused. Learn more about [resuming subscriptions](https://docs.stripe.com/docs/billing/subscriptions/pause#resume-subscriptions).
        """
        return cast(
            "Subscription",
            await self._request_async(
                "post",
                "/v1/subscriptions/{subscription}/resume".format(
                    subscription=sanitize_id(subscription),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def serialize_batch_cancel(
        self,
        subscription_exposed_id: str,
        params: Optional["SubscriptionCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> str:
        """
        Serializes a Subscription cancel request into a batch job JSONL line.
        """
        item_id = str(uuid4())
        stripe_version = (
            options.get("stripe_version") if options else None
        ) or _ApiVersion.CURRENT
        context = options.get("stripe_context") if options else None
        batch_request = {
            "id": item_id,
            "path_params": {
                "subscription_exposed_id": subscription_exposed_id
            },
            "params": params,
            "stripe_version": stripe_version,
        }
        if context is not None:
            batch_request["context"] = context
        return json.dumps(batch_request)

    def serialize_batch_update(
        self,
        subscription_exposed_id: str,
        params: Optional["SubscriptionUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> str:
        """
        Serializes a Subscription update request into a batch job JSONL line.
        """
        item_id = str(uuid4())
        stripe_version = (
            options.get("stripe_version") if options else None
        ) or _ApiVersion.CURRENT
        context = options.get("stripe_context") if options else None
        batch_request = {
            "id": item_id,
            "path_params": {
                "subscription_exposed_id": subscription_exposed_id
            },
            "params": params,
            "stripe_version": stripe_version,
        }
        if context is not None:
            batch_request["context"] = context
        return json.dumps(batch_request)

    def serialize_batch_create(
        self,
        params: Optional["SubscriptionCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> str:
        """
        Serializes a Subscription create request into a batch job JSONL line.
        """
        item_id = str(uuid4())
        stripe_version = (
            options.get("stripe_version") if options else None
        ) or _ApiVersion.CURRENT
        context = options.get("stripe_context") if options else None
        batch_request = {
            "id": item_id,
            "params": params,
            "stripe_version": stripe_version,
        }
        if context is not None:
            batch_request["context"] = context
        return json.dumps(batch_request)

    def serialize_batch_migrate(
        self,
        subscription: str,
        params: Optional["SubscriptionMigrateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> str:
        """
        Serializes a Subscription migrate request into a batch job JSONL line.
        """
        item_id = str(uuid4())
        stripe_version = (
            options.get("stripe_version") if options else None
        ) or _ApiVersion.CURRENT
        context = options.get("stripe_context") if options else None
        batch_request = {
            "id": item_id,
            "path_params": {"subscription": subscription},
            "params": params,
            "stripe_version": stripe_version,
        }
        if context is not None:
            batch_request["context"] = context
        return json.dumps(batch_request)

    def serialize_batch_pause(
        self,
        subscription: str,
        params: Optional["SubscriptionPauseParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> str:
        """
        Serializes a Subscription pause request into a batch job JSONL line.
        """
        item_id = str(uuid4())
        stripe_version = (
            options.get("stripe_version") if options else None
        ) or _ApiVersion.CURRENT
        context = options.get("stripe_context") if options else None
        batch_request = {
            "id": item_id,
            "path_params": {"subscription": subscription},
            "params": params,
            "stripe_version": stripe_version,
        }
        if context is not None:
            batch_request["context"] = context
        return json.dumps(batch_request)

    def serialize_batch_resume(
        self,
        subscription: str,
        params: Optional["SubscriptionResumeParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> str:
        """
        Serializes a Subscription resume request into a batch job JSONL line.
        """
        item_id = str(uuid4())
        stripe_version = (
            options.get("stripe_version") if options else None
        ) or _ApiVersion.CURRENT
        context = options.get("stripe_context") if options else None
        batch_request = {
            "id": item_id,
            "path_params": {"subscription": subscription},
            "params": params,
            "stripe_version": stripe_version,
        }
        if context is not None:
            batch_request["context"] = context
        return json.dumps(batch_request)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject, UntypedStripeObject
from typing import Any, ClassVar, Optional
from typing_extensions import Literal


class WorkflowRun(StripeObject):
    """
    An execution of a Workflow in response to a triggering event.
    """

    OBJECT_NAME: ClassVar[Literal["v2.extend.workflow_run"]] = (
        "v2.extend.workflow_run"
    )

    class StatusDetails(StripeObject):
        class Failed(StripeObject):
            error_message: Optional[str]
            """
            Optional details about the failure result.
            """

        class Started(StripeObject):
            pass

        class Succeeded(StripeObject):
            pass

        failed: Optional[Failed]
        """
        Details about the Workflow Run's transition into the FAILED state.
        """
        started: Optional[Started]
        """
        Details about the Workflow Run's transition in to the STARTED state.
        """
        succeeded: Optional[Succeeded]
        """
        Details about the Workflow Run's transition into the SUCCEEDED state.
        """
        _inner_class_types = {
            "failed": Failed,
            "started": Started,
            "succeeded": Succeeded,
        }

    class StatusTransitions(StripeObject):
        failed_at: Optional[str]
        """
        When the Workflow Run failed.
        """
        started_at: Optional[str]
        """
        When the Workflow Run was started.
        """
        succeeded_at: Optional[str]
        """
        When the Workflow Run succeeded.
        """

    class Trigger(StripeObject):
        class EventTrigger(StripeObject):
            context: str
            """
            The account that generated the triggering event.
            """
            id: str
            """
            The Stripe event that triggered this Run.
            """
            type: str
            """
            The Stripe event type triggered this Run.
            """

        class Manual(StripeObject):
            input_parameters: UntypedStripeObject[Any]
            """
            The input parameters used when launching the Run.
            """

        event_trigger: Optional[EventTrigger]
        """
        The Workflow Run was launched when Stripe emitted a certain event.
        """
        manual: Optional[Manual]
        """
        The Workflow Run was launched through a direct call, using either the Dashboard or the Stripe API.
        """
        type: Literal["event_trigger", "manual"]
        """
        Which type of trigger this is.
        """
        _inner_class_types = {"event_trigger": EventTrigger, "manual": Manual}

    created: str
    """
    When the Workflow Run was created.
    """
    id: str
    """
    The unique ID of the Workflow Run.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.extend.workflow_run"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    status: Literal["failed", "started", "succeeded"]
    """
    The current Workflow Run execution status.
    """
    status_details: Optional[StatusDetails]
    """
    Details about the Workflow Run's status transitions.
    """
    status_transitions: StatusTransitions
    """
    Summary information about the Workflow Run's status transitions.
    """
    trigger: Trigger
    """
    A record of the trigger that launched this Workflow Run.
    """
    workflow: str
    """
    The Workflow this Run belongs to.
    """
    _inner_class_types = {
        "status_details": StatusDetails,
        "status_transitions": StatusTransitions,
        "trigger": Trigger,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import Literal, NotRequired


class FinancingOfferRefillParams(RequestOptions):
    advance_amount: int
    """
    Amount of financing offered, in minor units. For example, 1,000 USD is represented as 100000.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    fee_amount: int
    """
    Fixed fee amount, in minor units. For example, 100 USD is represented as 10000.
    """
    financing_type: Literal["cash_advance", "fixed_term_loan", "flex_loan"]
    """
    The type of financing offer
    """
    status: NotRequired[
        Literal[
            "accepted",
            "accepted_other_offer",
            "canceled",
            "completed",
            "delivered",
            "expired",
            "fully_repaid",
            "paid_out",
            "rejected",
            "replaced",
            "undelivered",
        ]
    ]
    """
    The status of the financing offer
    """
    withhold_rate: float
    """
    Per-transaction rate at which Stripe withholds funds to repay the financing.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.tax._manual_rule_create_params import (
        ManualRuleCreateParams,
    )
    from stripe.params.v2.tax._manual_rule_deactivate_params import (
        ManualRuleDeactivateParams,
    )
    from stripe.params.v2.tax._manual_rule_list_params import (
        ManualRuleListParams,
    )
    from stripe.params.v2.tax._manual_rule_retrieve_params import (
        ManualRuleRetrieveParams,
    )
    from stripe.params.v2.tax._manual_rule_update_params import (
        ManualRuleUpdateParams,
    )
    from stripe.v2._list_object import ListObject
    from stripe.v2.tax._manual_rule import ManualRule


class ManualRuleService(StripeService):
    def list(
        self,
        params: Optional["ManualRuleListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[ManualRule]":
        """
        List all ManualRule objects.
        """
        return cast(
            "ListObject[ManualRule]",
            self._request(
                "get",
                "/v2/tax/manual_rules",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["ManualRuleListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[ManualRule]":
        """
        List all ManualRule objects.
        """
        return cast(
            "ListObject[ManualRule]",
            await self._request_async(
                "get",
                "/v2/tax/manual_rules",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "ManualRuleCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ManualRule":
        """
        Creates a ManualRule object.
        """
        return cast(
            "ManualRule",
            self._request(
                "post",
                "/v2/tax/manual_rules",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "ManualRuleCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ManualRule":
        """
        Creates a ManualRule object.
        """
        return cast(
            "ManualRule",
            await self._request_async(
                "post",
                "/v2/tax/manual_rules",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["ManualRuleRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ManualRule":
        """
        Retrieves a ManualRule object by ID.
        """
        return cast(
            "ManualRule",
            self._request(
                "get",
                "/v2/tax/manual_rules/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["ManualRuleRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ManualRule":
        """
        Retrieves a ManualRule object by ID.
        """
        return cast(
            "ManualRule",
            await self._request_async(
                "get",
                "/v2/tax/manual_rules/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        id: str,
        params: "ManualRuleUpdateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ManualRule":
        """
        Updates the Tax configuration for a ManualRule object.
        """
        return cast(
            "ManualRule",
            self._request(
                "post",
                "/v2/tax/manual_rules/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        id: str,
        params: "ManualRuleUpdateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ManualRule":
        """
        Updates the Tax configuration for a ManualRule object.
        """
        return cast(
            "ManualRule",
            await self._request_async(
                "post",
                "/v2/tax/manual_rules/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def deactivate(
        self,
        id: str,
        params: Optional["ManualRuleDeactivateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ManualRule":
        """
        Deactivates a ManualRule object.
        """
        return cast(
            "ManualRule",
            self._request(
                "post",
                "/v2/tax/manual_rules/{id}/deactivate".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def deactivate_async(
        self,
        id: str,
        params: Optional["ManualRuleDeactivateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ManualRule":
        """
        Deactivates a ManualRule object.
        """
        return cast(
            "ManualRule",
            await self._request_async(
                "post",
                "/v2/tax/manual_rules/{id}/deactivate".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject, UntypedStripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal


class PricingPlan(StripeObject):
    """
    A Pricing Plan represents a bundled collection of billing components that define how customers are charged. Each plan
    can include multiple components such as Rate Cards for usage-based pricing, License Fees for recurring charges, and Service
    Actions for recurring credit grants. After creating a Pricing Plan, you can subscribe customers to it by creating a Pricing Plan Subscription.
    """

    OBJECT_NAME: ClassVar[Literal["v2.billing.pricing_plan"]] = (
        "v2.billing.pricing_plan"
    )
    active: bool
    """
    Whether the PricingPlan is active.
    """
    created: str
    """
    Time at which the object was created.
    """
    currency: str
    """
    The currency of the PricingPlan.
    """
    description: Optional[str]
    """
    A description of the pricing plan. Maximum length of 500 characters.
    """
    display_name: str
    """
    Display name of the PricingPlan.
    """
    id: str
    """
    Unique identifier for the object.
    """
    live_version: Optional[str]
    """
    The ID of the live version of the PricingPlan.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    lookup_key: Optional[str]
    """
    An internal key you can use to search for a particular PricingPlan. Maximum length of 200 characters.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of [key-value pairs](https://docs.stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["v2.billing.pricing_plan"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    tax_behavior: Literal["exclusive", "inclusive"]
    """
    The tax behavior for Stripe Tax — whether the pricing plan price includes or excludes tax.
    """
